<?php

namespace Drupal\fivestar\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'fivestar_select' widget.
 *
 * @FieldWidget(
 *   id = "fivestar_select",
 *   label = @Translation("Select list"),
 *   field_types = {
 *     "fivestar"
 *   }
 * )
 */
class SelectWidget extends FiveStartWidgetBase {

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $settings = $items[$delta]->getFieldDefinition()
      ->getSettings();

    $options = [];
    for ($i = 1; $i <= $settings['stars']; $i++) {
      $this_value = ceil($i * 100 / $settings['stars']);
      $options[$this_value] = $this->t('Give @star/@count', [
        '@star' => $i,
        '@count' => $settings['stars'],
      ]);
    }

    $element += [
      '#type' => 'item',
    ];

    $element['rating'] = [
      '#type' => 'select',
      '#empty_option' => $this->t('Select rating:'),
      '#empty_value' => '-',
      '#options' => $options,
      '#required' => $items[$delta]->getFieldDefinition()->isRequired(),
      '#default_value' => isset($items[$delta]->rating) ? $items[$delta]->rating : 0,
    ];

    return $element;
  }

}
